@@ -17,8 +17,10 @@ from __future__ import annotations
 import fcntl
 import logging
 import os
+import random
+import time
 from collections.abc import Iterator
-from contextlib import contextmanager, suppress
+from contextlib import contextmanager
 from datetime import UTC, datetime
 from pathlib import Path
 from typing import TYPE_CHECKING
@@ -36,10 +38,14 @@ from git_notes_memory.exceptions import (
 from git_notes_memory.git_ops import GitOps
 from git_notes_memory.models import CaptureResult, Memory
 from git_notes_memory.note_parser import serialize_note
+from git_notes_memory.observability.decorators import measure_duration
+from git_notes_memory.observability.metrics import get_metrics
+from git_notes_memory.observability.tracing import trace_operation
 
 if TYPE_CHECKING:
     from git_notes_memory.embedding import EmbeddingService
     from git_notes_memory.index import IndexService
+    from git_notes_memory.security.service import SecretsFilteringService
 
 __all__ = [
     "CaptureService",
@@ -55,51 +61,85 @@ logger = logging.getLogger(__name__)
 
 
 @contextmanager
-def _acquire_lock(lock_path: Path, _timeout: float = 10.0) -> Iterator[None]:
+def _acquire_lock(lock_path: Path, timeout: float = 10.0) -> Iterator[None]:
     """Acquire an exclusive file lock for capture operations.
 
     Uses fcntl advisory locking to prevent concurrent corruption. The lock
     is automatically released when the context manager exits.
 
+    Uses non-blocking lock with retry loop to implement timeout, preventing
+    indefinite blocking if another process holds the lock.
+
     Args:
         lock_path: Path to the lock file.
-        _timeout: Maximum time to wait for lock (seconds). Currently not
-            implemented - will block indefinitely if lock is held.
+        timeout: Maximum time to wait for lock (seconds). Default 10.0.
 
     Yields:
         None when lock is acquired.
 
     Raises:
-        CaptureError: If the lock cannot be acquired.
+        CaptureError: If the lock cannot be acquired within the timeout.
     """
     # Ensure parent directory exists
     lock_path.parent.mkdir(parents=True, exist_ok=True)
 
     fd = None
     try:
-        # Open or create lock file
-        fd = os.open(str(lock_path), os.O_RDWR | os.O_CREAT, 0o644)
+        # Open or create lock file with restrictive permissions (MED-001)
+        # O_NOFOLLOW prevents symlink attacks (HIGH-005: TOCTOU mitigation)
+        fd = os.open(
+            str(lock_path),
+            os.O_RDWR | os.O_CREAT | os.O_NOFOLLOW,
+            0o600,
+        )
+
+        # Acquire exclusive lock with timeout using non-blocking retry loop
+        # CRIT-001: Prevents indefinite blocking if lock is held
+        # Uses exponential backoff with jitter to reduce contention under high concurrency
+        deadline = time.monotonic() + timeout
+        base_interval = 0.05  # Start with 50ms
+        max_interval = 2.0  # Cap at 2 seconds
+        attempt = 0
+
+        while True:
+            try:
+                fcntl.flock(fd, fcntl.LOCK_EX | fcntl.LOCK_NB)
+                logger.debug("Acquired capture lock: %s", lock_path)
+                break
+            except BlockingIOError:
+                if time.monotonic() >= deadline:
+                    raise CaptureError(
+                        f"Lock acquisition timed out after {timeout}s",
+                        "Another capture may be in progress, wait and retry",
+                    ) from None
+                # Exponential backoff: 50ms, 100ms, 200ms, ... up to max_interval
+                # Add jitter (0-10% of interval) to prevent thundering herd
+                interval = min(base_interval * (2**attempt), max_interval)
+                jitter = random.uniform(0, interval * 0.1)  # noqa: S311
+                time.sleep(interval + jitter)
+                attempt += 1
+            except OSError as e:
+                raise CaptureError(
+                    f"Failed to acquire capture lock: {e}",
+                    "Another capture may be in progress, wait and retry",
+                ) from e
+
+        yield
 
-        # Acquire exclusive lock (blocking)
-        # Note: timeout is not implemented with fcntl, it blocks indefinitely
-        # A more sophisticated implementation could use LOCK_NB with retries
-        try:
-            fcntl.flock(fd, fcntl.LOCK_EX)
-            logger.debug("Acquired capture lock: %s", lock_path)
-            yield
-        except OSError as e:
-            raise CaptureError(
-                f"Failed to acquire capture lock: {e}",
-                "Another capture may be in progress, wait and retry",
-            ) from e
     finally:
         if fd is not None:
             try:
                 fcntl.flock(fd, fcntl.LOCK_UN)
                 os.close(fd)
                 logger.debug("Released capture lock: %s", lock_path)
-            except OSError:
-                pass  # Best effort cleanup
+            except OSError as e:
+                # Log warning rather than silently ignoring - helps debug lock issues
+                logger.warning("Failed to release capture lock %s: %s", lock_path, e)
+                metrics = get_metrics()
+                metrics.increment(
+                    "silent_failures_total",
+                    labels={"location": "capture.lock_release"},
+                )
 
 
 # =============================================================================
@@ -202,6 +242,7 @@ class CaptureService:
         git_ops: GitOps | None = None,
         index_service: IndexService | None = None,
         embedding_service: EmbeddingService | None = None,
+        secrets_service: SecretsFilteringService | None = None,
         repo_path: Path | None = None,
     ) -> None:
         """Initialize the capture service.
@@ -213,12 +254,15 @@ class CaptureService:
                 are captured but not indexed.
             embedding_service: EmbeddingService for embeddings. If None,
                 memories are indexed without embeddings.
+            secrets_service: SecretsFilteringService for filtering sensitive
+                data. If None, content is captured without filtering.
             repo_path: Path to the git repository. Only used if git_ops
                 is None.
         """
         self._git_ops = git_ops
         self._index_service = index_service
         self._embedding_service = embedding_service
+        self._secrets_service: SecretsFilteringService | None = secrets_service
         self._repo_path = repo_path
         self._lock_path = get_lock_path()
 
@@ -253,10 +297,149 @@ class CaptureService:
         """
         self._embedding_service = service
 
+    @property
+    def secrets_service(self) -> SecretsFilteringService | None:
+        """Get the SecretsFilteringService instance."""
+        return self._secrets_service
+
+    def set_secrets_service(self, service: SecretsFilteringService) -> None:
+        """Set the secrets filtering service after initialization.
+
+        Useful for lazy initialization or dependency injection.
+        """
+        self._secrets_service = service
+
+    # =========================================================================
+    # Input Validation (extracted for ARCH-007)
+    # =========================================================================
+
+    def _validate_capture_input(
+        self,
+        namespace: str,
+        summary: str,
+        content: str,
+    ) -> None:
+        """Validate all capture input parameters.
+
+        Args:
+            namespace: Memory type to validate.
+            summary: One-line summary to validate.
+            content: Full content to validate.
+
+        Raises:
+            ValidationError: If any validation fails.
+        """
+        _validate_namespace(namespace)
+        _validate_summary(summary)
+        _validate_content(content)
+
+    def _filter_content(
+        self,
+        summary: str,
+        content: str,
+        namespace: str,
+    ) -> tuple[str, str, list[str]]:
+        """Filter secrets from summary and content.
+
+        Applies secrets filtering if a SecretsFilteringService is configured.
+        Raises BlockedContentError if content contains blocked secrets.
+
+        Args:
+            summary: The one-line summary to filter.
+            content: The full content to filter.
+            namespace: Memory namespace for allowlist scoping.
+
+        Returns:
+            Tuple of (filtered_summary, filtered_content, warnings).
+
+        Raises:
+            BlockedContentError: If BLOCK strategy is configured and secrets found.
+        """
+        if self._secrets_service is None or not self._secrets_service.enabled:
+            return summary, content, []
+
+        warnings: list[str] = []
+
+        # Filter summary
+        summary_result = self._secrets_service.filter(
+            summary,
+            source="capture_summary",
+            namespace=namespace,
+        )
+        filtered_summary = summary_result.content
+        if summary_result.warnings:
+            warnings.extend(summary_result.warnings)
+        if summary_result.had_secrets:
+            warnings.append(
+                f"Summary contained {summary_result.detection_count} secret(s), redacted"
+            )
+
+        # Filter content
+        content_result = self._secrets_service.filter(
+            content,
+            source="capture_content",
+            namespace=namespace,
+        )
+        filtered_content = content_result.content
+        if content_result.warnings:
+            warnings.extend(content_result.warnings)
+        if content_result.had_secrets:
+            warnings.append(
+                f"Content contained {content_result.detection_count} secret(s), redacted"
+            )
+
+        return filtered_summary, filtered_content, warnings
+
+    def _build_front_matter(
+        self,
+        namespace: str,
+        summary: str,
+        timestamp: datetime,
+        spec: str | None,
+        phase: str | None,
+        tags: tuple[str, ...],
+        status: str,
+        relates_to: tuple[str, ...],
+    ) -> dict[str, object]:
+        """Build the YAML front matter dictionary.
+
+        Args:
+            namespace: Memory type.
+            summary: One-line summary.
+            timestamp: Capture timestamp.
+            spec: Optional specification slug.
+            phase: Optional lifecycle phase.
+            tags: Categorization tags.
+            status: Memory status.
+            relates_to: Related memory IDs.
+
+        Returns:
+            Dictionary ready for YAML serialization.
+        """
+        front_matter: dict[str, object] = {
+            "type": namespace,
+            "timestamp": timestamp.isoformat(),
+            "summary": summary,
+        }
+
+        if spec:
+            front_matter["spec"] = spec
+        if phase:
+            front_matter["phase"] = phase
+        if tags:
+            front_matter["tags"] = list(tags)
+        if status != "active":
+            front_matter["status"] = status
+        if relates_to:
+            front_matter["relates_to"] = list(relates_to)
+
+        return front_matter
+
     # =========================================================================
     # Core Capture Method
     # =========================================================================
 
+    @measure_duration("memory_capture")
     def capture(
         self,
         namespace: str,
@@ -307,10 +490,14 @@ class CaptureService:
             >>> result.success
             True
         """
-        # Validate input
-        _validate_namespace(namespace)
-        _validate_summary(summary)
-        _validate_content(content)
+        # Validate input (extracted method for ARCH-007)
+        self._validate_capture_input(namespace, summary, content)
+
+        # Filter secrets from summary and content
+        # This may raise BlockedContentError if BLOCK strategy is configured
+        filtered_summary, filtered_content, filter_warnings = self._filter_content(
+            summary, content, namespace
+        )
 
         # Normalize tags
         tags_tuple = tuple(tags) if tags else ()
@@ -319,33 +506,29 @@ class CaptureService:
         # Get current timestamp
         timestamp = datetime.now(UTC)
 
-        # Build front matter
-        front_matter: dict[str, object] = {
-            "type": namespace,
-            "timestamp": timestamp.isoformat(),
-            "summary": summary,
-        }
-
-        if spec:
-            front_matter["spec"] = spec
-        if phase:
-            front_matter["phase"] = phase
-        if tags_tuple:
-            front_matter["tags"] = list(tags_tuple)
-        if status != "active":
-            front_matter["status"] = status
-        if relates_tuple:
-            front_matter["relates_to"] = list(relates_tuple)
+        # Build front matter (extracted method for ARCH-007)
+        # Use filtered_summary in front matter
+        front_matter = self._build_front_matter(
+            namespace=namespace,
+            summary=filtered_summary,
+            timestamp=timestamp,
+            spec=spec,
+            phase=phase,
+            tags=tags_tuple,
+            status=status,
+            relates_to=relates_tuple,
+        )
 
         # Serialize to YAML front matter format
-        note_content = serialize_note(front_matter, content)
+        # Use filtered_content in note body
+        note_content = serialize_note(front_matter, filtered_content)
 
         # Capture with or without locking
         if skip_lock:
             return self._do_capture(
                 namespace=namespace,
-                summary=summary,
-                content=content,
+                summary=filtered_summary,
+                content=filtered_content,
                 note_content=note_content,
                 timestamp=timestamp,
                 spec=spec,
@@ -354,13 +537,14 @@ class CaptureService:
                 status=status,
                 relates_to=relates_tuple,
                 commit=commit,
+                filter_warnings=filter_warnings,
             )
 
         with _acquire_lock(self._lock_path):
             return self._do_capture(
                 namespace=namespace,
-                summary=summary,
-                content=content,
+                summary=filtered_summary,
+                content=filtered_content,
                 note_content=note_content,
                 timestamp=timestamp,
                 spec=spec,
@@ -369,6 +553,7 @@ class CaptureService:
                 status=status,
                 relates_to=relates_tuple,
                 commit=commit,
+                filter_warnings=filter_warnings,
             )
 
     def _do_capture(
@@ -385,91 +570,124 @@ class CaptureService:
         status: str,
         relates_to: tuple[str, ...],
         commit: str,
+        filter_warnings: list[str] | None = None,
     ) -> CaptureResult:
         """Execute the capture operation (called within lock).
 
         Internal method that performs the actual capture work.
+
+        Args:
+            filter_warnings: Optional list of warnings from secrets filtering.
         """
-        # Resolve commit SHA
-        try:
-            commit_info = self.git_ops.get_commit_info(commit)
-            commit_sha = commit_info.sha
-        except Exception as e:
-            raise CaptureError(
-                f"Failed to resolve commit '{commit}': {e}",
-                "Ensure you're in a git repository with valid commits",
-            ) from e
+        metrics = get_metrics()
 
-        # Determine note index (count existing notes by "---" pairs in this namespace)
-        try:
-            existing_note = self.git_ops.show_note(namespace, commit_sha)
-            index = existing_note.count("\n---\n") // 2 + 1 if existing_note else 0
-        except Exception:
-            index = 0
+        with trace_operation("capture", labels={"namespace": namespace}):
+            # Resolve commit SHA
+            with trace_operation("capture.resolve_commit"):
+                try:
+                    commit_info = self.git_ops.get_commit_info(commit)
+                    commit_sha = commit_info.sha
+                except Exception as e:
+                    raise CaptureError(
+                        f"Failed to resolve commit '{commit}': {e}",
+                        "Ensure you're in a git repository with valid commits",
+                    ) from e
 
-        # Build memory ID
-        memory_id = f"{namespace}:{commit_sha}:{index}"
+            # Determine note index (count existing notes by "---" pairs)
+            with trace_operation("capture.count_existing"):
+                try:
+                    existing_note = self.git_ops.show_note(namespace, commit_sha)
+                    index = (
+                        existing_note.count("\n---\n") // 2 + 1 if existing_note else 0
+                    )
+                except Exception as e:
+                    logger.warning(
+                        "Failed to count existing notes for %s:%s: %s",
+                        namespace,
+                        commit_sha[:8],
+                        e,
+                    )
+                    metrics.increment(
+                        "silent_failures_total",
+                        labels={"location": "capture.count_existing"},
+                    )
+                    index = 0
 
-        # Write to git notes (append for safety)
-        try:
-            self.git_ops.append_note(namespace, note_content, commit_sha)
-            logger.info("Captured memory: %s", memory_id)
-        except Exception as e:
-            raise CaptureError(
-                f"Failed to write git note: {e}",
-                "Check git repository status and permissions",
-            ) from e
+            # Build memory ID
+            memory_id = f"{namespace}:{commit_sha}:{index}"
 
-        # Create Memory object
-        memory = Memory(
-            id=memory_id,
-            commit_sha=commit_sha,
-            namespace=namespace,
-            summary=summary,
-            content=content,
-            timestamp=timestamp,
-            spec=spec,
-            phase=phase,
-            tags=tags,
-            status=status,
-            relates_to=relates_to,
-        )
+            # Write to git notes (append for safety)
+            with trace_operation("capture.git_append"):
+                try:
+                    self.git_ops.append_note(namespace, note_content, commit_sha)
+                    logger.info("Captured memory: %s", memory_id)
+                except Exception as e:
+                    raise CaptureError(
+                        f"Failed to write git note: {e}",
+                        "Check git repository status and permissions",
+                    ) from e
 
-        # Try to index (graceful degradation)
-        indexed = False
-        warning: str | None = None
+            # Create Memory object
+            memory = Memory(
+                id=memory_id,
+                commit_sha=commit_sha,
+                namespace=namespace,
+                summary=summary,
+                content=content,
+                timestamp=timestamp,
+                spec=spec,
+                phase=phase,
+                tags=tags,
+                status=status,
+                relates_to=relates_to,
+            )
 
-        if self._index_service is not None:
-            try:
-                # Generate embedding if service available
-                embedding: list[float] | None = None
-                if self._embedding_service is not None:
+            # Try to index (graceful degradation)
+            # Initialize warnings list with any filter warnings from secrets filtering
+            indexed = False
+            warnings: list[str] = list(filter_warnings) if filter_warnings else []
+
+            if self._index_service is not None:
+                with trace_operation("capture.index"):
                     try:
-                        # Combine summary and content for embedding
-                        embed_text = f"{summary}\n\n{content}"
-                        embedding = self._embedding_service.embed(embed_text)
+                        # Generate embedding if service available
+                        embedding: list[float] | None = None
+                        if self._embedding_service is not None:
+                            with trace_operation("capture.embed"):
+                                try:
+                                    # Combine summary and content for embedding
+                                    embed_text = f"{summary}\n\n{content}"
+                                    embedding = self._embedding_service.embed(
+                                        embed_text
+                                    )
+                                except Exception as e:
+                                    warnings.append(f"Embedding failed: {e}")
+                                    logger.warning("Embedding generation failed: %s", e)
+
+                        # Insert into index
+                        self._index_service.insert(memory, embedding)
+                        indexed = True
+                        logger.debug("Indexed memory: %s", memory_id)
+
                     except Exception as e:
-                        warning = f"Embedding failed: {e}"
-                        logger.warning("Embedding generation failed: %s", e)
+                        warnings.append(f"Indexing failed: {e}")
+                        logger.warning("Indexing failed for %s: %s", memory_id, e)
 
-                # Insert into index
-                self._index_service.insert(memory, embedding)
-                indexed = True
-                logger.debug("Indexed memory: %s", memory_id)
+            # Increment counter on successful capture
+            metrics.increment(
+                "memories_captured_total",
+                labels={"namespace": namespace},
+            )
 
-            except Exception as e:
-                if warning:
-                    warning = f"{warning}; Indexing failed: {e}"
-                else:
-                    warning = f"Indexing failed: {e}"
-                logger.warning("Indexing failed for %s: %s", memory_id, e)
+            # Combine all warnings into a single string (or None if no warnings)
+            combined_warning = "; ".join(warnings) if warnings else None
 
-        return CaptureResult(
-            success=True,
-            memory=memory,
-            indexed=indexed,
-            warning=warning,
-        )
+            return CaptureResult(
+                success=True,
+                memory=memory,
+                indexed=indexed,
+                warning=combined_warning,
+            )
 
     # =========================================================================
     # Convenience Capture Methods
@@ -898,13 +1116,10 @@ class CaptureService:
 
 
 # =============================================================================
-# Singleton Instance
+# Singleton Access (using ServiceRegistry)
 # =============================================================================
 
 
-_default_service: CaptureService | None = None
-
-
 def get_default_service() -> CaptureService:
     """Get the default capture service singleton.
 
@@ -915,14 +1130,15 @@ def get_default_service() -> CaptureService:
         The default service is created without index or embedding services.
         Use set_index_service() and set_embedding_service() to enable
         these features after getting the default service.
-
-        On first creation, also ensures git notes sync is configured
-        for the repository (push/fetch refspecs for notes).
     """
-    global _default_service
-    if _default_service is None:
-        _default_service = CaptureService()
-        # Ensure git notes sync is configured on first use (best effort)
-        with suppress(Exception):
-            _default_service.git_ops.ensure_sync_configured()
-    return _default_service
+    from git_notes_memory.registry import ServiceRegistry
+
+    service = ServiceRegistry.get(CaptureService)
+
+    # Ensure git notes sync is configured on first use (best effort)
+    try:
+        service.git_ops.ensure_sync_configured()
+    except Exception as e:
+        logger.debug("Git notes sync auto-configuration skipped: %s", e)
+
+    return service
