@@ -14,11 +14,11 @@ The context is structured into:
 
 from __future__ import annotations
 
-import logging
 from datetime import UTC, datetime, timedelta
 from typing import TYPE_CHECKING
 
 from git_notes_memory.config import TOKENS_PER_CHAR, get_project_index_path
+from git_notes_memory.exceptions import MemoryIndexError
 from git_notes_memory.hooks.config_loader import (
     BudgetMode,
     HookConfig,
@@ -31,6 +31,7 @@ from git_notes_memory.hooks.models import (
     WorkingMemory,
 )
 from git_notes_memory.hooks.xml_formatter import XMLBuilder
+from git_notes_memory.observability import get_logger
 
 if TYPE_CHECKING:
     from git_notes_memory.index import IndexService
@@ -39,7 +40,7 @@ if TYPE_CHECKING:
 
 __all__ = ["ContextBuilder"]
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 class ContextBuilder:
@@ -556,7 +557,8 @@ class ContextBuilder:
                 return "complex"
             return "full"
 
-        except Exception as e:
+        # QUAL-002: Catch specific exceptions instead of bare Exception
+        except (MemoryIndexError, OSError) as e:
             logger.debug("Failed to analyze complexity for %s: %s", project, e)
             return "medium"  # Default to medium on error
 
