@@ -20,15 +20,19 @@ Example::
 
 from __future__ import annotations
 
-import logging
 import re
 from typing import ClassVar
 
 from git_notes_memory.hooks.models import CaptureSignal, SignalType
+from git_notes_memory.observability import get_logger
 
 __all__ = ["SignalDetector", "SIGNAL_PATTERNS"]
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
+
+# SEC-001: Maximum text length for signal detection to prevent ReDoS attacks
+# 100KB is generous for user prompts while preventing abuse
+MAX_TEXT_LENGTH = 100 * 1024
 
 # Pattern definitions by signal type
 # Each pattern is a tuple of (regex_string, base_confidence)
@@ -271,6 +275,15 @@ class SignalDetector:
         if not text or len(text) < 5:
             return []
 
+        # SEC-001: Limit input length to prevent ReDoS attacks
+        if len(text) > MAX_TEXT_LENGTH:
+            logger.warning(
+                "Input text length %d exceeds maximum %d, truncating for safety",
+                len(text),
+                MAX_TEXT_LENGTH,
+            )
+            text = text[:MAX_TEXT_LENGTH]
+
         signals: list[CaptureSignal] = []
         block_positions: set[tuple[int, int]] = set()
 
