@@ -28,13 +28,13 @@ Environment Variables:
     HOOK_STOP_ENABLED: Enable this hook (default: true)
     HOOK_STOP_PROMPT_UNCAPTURED: Prompt for uncaptured content (default: true)
     HOOK_STOP_SYNC_INDEX: Sync index on session end (default: true)
+    HOOK_STOP_PUSH_REMOTE: Push notes to remote on stop (default: false)
     HOOK_DEBUG: Enable debug logging (default: false)
 """
 
 from __future__ import annotations
 
 import json
-import logging
 import sys
 from pathlib import Path
 from typing import Any
@@ -45,33 +45,45 @@ from git_notes_memory.hooks.hook_utils import (
     cancel_timeout,
     get_hook_logger,
     log_hook_input,
+    read_json_input,
     setup_logging,
     setup_timeout,
+    timed_hook_execution,
 )
 from git_notes_memory.hooks.models import CaptureSignal
+from git_notes_memory.observability import get_logger
+from git_notes_memory.observability.exporters.otlp import (
+    export_metrics_if_configured,
+    export_traces_if_configured,
+)
+from git_notes_memory.observability.tracing import (
+    clear_completed_spans,
+    get_completed_spans,
+)
 
 __all__ = ["main"]
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
-def _read_input() -> dict[str, Any]:
-    """Read and parse JSON input from stdin.
+def _read_input_with_fallback() -> dict[str, Any]:
+    """Read and parse JSON input from stdin with fallback for empty input.
+
+    QUAL-001: Wraps hook_utils.read_json_input() with Stop-hook-specific
+    fallback behavior (empty input is valid for stop hooks).
 
     Returns:
-        Parsed JSON data.
-
-    Raises:
-        json.JSONDecodeError: If input is not valid JSON.
+        Parsed JSON data, or empty dict if stdin is empty.
     """
-    input_text = sys.stdin.read()
-    if not input_text.strip():
+    try:
+        return read_json_input()
+    except ValueError as e:
         # Empty input is OK for stop hook
-        return {}
-    result = json.loads(input_text)
-    if not isinstance(result, dict):
-        return {}
-    return dict(result)
+        # Check for the specific error message from read_json_input
+        err_msg = str(e).lower()
+        if "empty input" in err_msg or "stdin" in err_msg and "empty" in err_msg:
+            return {}
+        raise
 
 
 def _analyze_session(transcript_path: str | None) -> list[CaptureSignal]:
@@ -229,6 +241,43 @@ def _auto_capture_signals(
     return captured, remaining
 
 
+def _flush_telemetry() -> dict[str, Any]:
+    """Flush accumulated telemetry to OTLP endpoint.
+
+    Exports all collected traces and metrics to the configured OTLP
+    endpoint (if any). Called at session end to ensure telemetry is shipped.
+
+    Returns:
+        Dict with export results.
+    """
+    result: dict[str, Any] = {"traces": False, "metrics": False}
+
+    try:
+        # Export traces
+        spans = get_completed_spans()
+        if spans:
+            if export_traces_if_configured(spans):
+                result["traces"] = True
+                result["trace_count"] = len(spans)
+                clear_completed_spans()
+                logger.debug("Exported %d traces to OTLP", len(spans))
+            else:
+                logger.debug("Trace export skipped (no endpoint or failed)")
+
+        # Export metrics
+        if export_metrics_if_configured():
+            result["metrics"] = True
+            logger.debug("Exported metrics to OTLP")
+        else:
+            logger.debug("Metrics export skipped (no endpoint or failed)")
+
+    except Exception as e:
+        logger.debug("Telemetry flush error: %s", e)
+        result["error"] = str(e)
+
+    return result
+
+
 def _signal_to_dict(signal: CaptureSignal) -> dict[str, Any]:
     """Convert a CaptureSignal to a JSON-serializable dict.
 
@@ -384,104 +433,137 @@ def main() -> None:
     timeout = config.timeout or HOOK_STOP_TIMEOUT
     setup_timeout(timeout, hook_name="Stop")
 
-    try:
-        # Read input (may be empty for stop hook)
-        input_data = _read_input()
-        logger.debug("Received stop hook input: %s", list(input_data.keys()))
+    with timed_hook_execution("Stop") as timer:
+        try:
+            # QUAL-001: Use hook_utils.read_json_input with fallback
+            input_data = _read_input_with_fallback()
+            logger.debug("Received stop hook input: %s", list(input_data.keys()))
 
-        # Log full input to file for debugging
-        log_hook_input("Stop", input_data)
-        hook_logger.info(
-            "Config: stop_auto_capture=%s, stop_prompt_uncaptured=%s, stop_sync_index=%s",
-            config.stop_auto_capture,
-            config.stop_prompt_uncaptured,
-            config.stop_sync_index,
-        )
-
-        # Analyze session transcript for uncaptured content
-        detected_signals: list[CaptureSignal] = []
-        if config.stop_prompt_uncaptured or config.stop_auto_capture:
-            transcript_path = input_data.get("transcript_path")
-            hook_logger.info("Analyzing transcript: %s", transcript_path)
-            detected_signals = _analyze_session(transcript_path)
-            hook_logger.info("Found %d signals in transcript", len(detected_signals))
-            for sig in detected_signals[:5]:  # Log first 5
-                hook_logger.info(
-                    "  Signal: type=%s, ns=%s, conf=%.2f, match=%s...",
-                    sig.type.value,
-                    sig.suggested_namespace,
-                    sig.confidence,
-                    sig.match[:50],
-                )
-        else:
+            # Log full input to file for debugging
+            log_hook_input("Stop", input_data)
             hook_logger.info(
-                "Skipping transcript analysis (auto_capture=%s, prompt_uncaptured=%s)",
+                "Config: stop_auto_capture=%s, stop_prompt_uncaptured=%s, stop_sync_index=%s",
                 config.stop_auto_capture,
                 config.stop_prompt_uncaptured,
+                config.stop_sync_index,
             )
 
-        # Auto-capture high-confidence signals
-        captured: list[dict[str, Any]] = []
-        uncaptured: list[CaptureSignal] = detected_signals
-        if config.stop_auto_capture and detected_signals:
-            hook_logger.info(
-                "Auto-capturing signals (min_conf=%.2f, max=%d)",
-                config.stop_auto_capture_min_confidence,
-                config.stop_max_captures,
-            )
-            captured, uncaptured = _auto_capture_signals(
-                detected_signals,
-                min_confidence=config.stop_auto_capture_min_confidence,
-                max_captures=config.stop_max_captures,
-            )
-            hook_logger.info(
-                "Auto-capture result: %d captured, %d remaining",
-                len(captured),
-                len(uncaptured),
-            )
-            for c in captured:
-                hook_logger.info("  Captured: %s", c)
-            logger.debug(
-                "Auto-capture: %d captured, %d remaining",
-                len(captured),
-                len(uncaptured),
-            )
-        else:
-            hook_logger.info(
-                "Auto-capture skipped (auto_capture=%s, signals=%d)",
-                config.stop_auto_capture,
-                len(detected_signals),
-            )
-
-        # Sync index if enabled (after auto-capture to include new memories)
-        sync_result: dict[str, Any] | None = None
-        if config.stop_sync_index:
-            sync_result = _sync_index()
-            if sync_result.get("success") and not sync_result.get("skipped"):
-                stats = sync_result.get("stats", {})
-                logger.info(
-                    "Index synced: %d memories indexed",
-                    stats.get("indexed", 0),
+            # Analyze session transcript for uncaptured content
+            detected_signals: list[CaptureSignal] = []
+            if config.stop_prompt_uncaptured or config.stop_auto_capture:
+                transcript_path = input_data.get("transcript_path")
+                hook_logger.info("Analyzing transcript: %s", transcript_path)
+                detected_signals = _analyze_session(transcript_path)
+                hook_logger.info(
+                    "Found %d signals in transcript", len(detected_signals)
+                )
+                for sig in detected_signals[:5]:  # Log first 5
+                    hook_logger.info(
+                        "  Signal: type=%s, ns=%s, conf=%.2f, match=%s...",
+                        sig.type.value,
+                        sig.suggested_namespace,
+                        sig.confidence,
+                        sig.match[:50],
+                    )
+            else:
+                hook_logger.info(
+                    "Skipping transcript analysis (auto_capture=%s, prompt_uncaptured=%s)",
+                    config.stop_auto_capture,
+                    config.stop_prompt_uncaptured,
                 )
-            elif not sync_result.get("success"):
-                logger.warning("Index sync failed: %s", sync_result.get("error"))
 
-        # Output result
-        _write_output(
-            uncaptured=uncaptured,
-            captured=captured,
-            sync_result=sync_result,
-            prompt_uncaptured=config.stop_prompt_uncaptured,
-        )
+            # Auto-capture high-confidence signals
+            captured: list[dict[str, Any]] = []
+            uncaptured: list[CaptureSignal] = detected_signals
+            if config.stop_auto_capture and detected_signals:
+                hook_logger.info(
+                    "Auto-capturing signals (min_conf=%.2f, max=%d)",
+                    config.stop_auto_capture_min_confidence,
+                    config.stop_max_captures,
+                )
+                captured, uncaptured = _auto_capture_signals(
+                    detected_signals,
+                    min_confidence=config.stop_auto_capture_min_confidence,
+                    max_captures=config.stop_max_captures,
+                )
+                hook_logger.info(
+                    "Auto-capture result: %d captured, %d remaining",
+                    len(captured),
+                    len(uncaptured),
+                )
+                for c in captured:
+                    hook_logger.info("  Captured: %s", c)
+                logger.debug(
+                    "Auto-capture: %d captured, %d remaining",
+                    len(captured),
+                    len(uncaptured),
+                )
+            else:
+                hook_logger.info(
+                    "Auto-capture skipped (auto_capture=%s, signals=%d)",
+                    config.stop_auto_capture,
+                    len(detected_signals),
+                )
 
-    except json.JSONDecodeError as e:
-        logger.error("Failed to parse hook input: %s", e)
-        print(json.dumps({"continue": True}))
-    except Exception as e:
-        logger.exception("Stop hook error: %s", e)
-        print(json.dumps({"continue": True}))
-    finally:
-        cancel_timeout()
+            # Sync index if enabled (after auto-capture to include new memories)
+            sync_result: dict[str, Any] | None = None
+            if config.stop_sync_index:
+                sync_result = _sync_index()
+                if sync_result.get("success") and not sync_result.get("skipped"):
+                    stats = sync_result.get("stats", {})
+                    logger.info(
+                        "Index synced: %d memories indexed",
+                        stats.get("indexed", 0),
+                    )
+                elif not sync_result.get("success"):
+                    logger.warning("Index sync failed: %s", sync_result.get("error"))
+
+            # Push notes to remote if enabled (opt-in via env var)
+            # This ensures local memories are shared with collaborators
+            if config.stop_push_remote:
+                cwd = input_data.get("cwd")
+                if cwd:
+                    try:
+                        from git_notes_memory.git_ops import GitOps
+
+                        git_ops = GitOps(repo_path=cwd)
+                        if git_ops.push_notes_to_remote():
+                            logger.debug("Pushed notes to remote on session stop")
+                        else:
+                            logger.debug(
+                                "Push to remote failed (will retry next session)"
+                            )
+                    except Exception as e:
+                        logger.debug("Remote push on stop skipped: %s", e)
+
+            # Flush telemetry to OTLP endpoint (if configured)
+            telemetry_result = _flush_telemetry()
+            if telemetry_result.get("traces") or telemetry_result.get("metrics"):
+                hook_logger.info(
+                    "Telemetry flushed: traces=%s (count=%d), metrics=%s",
+                    telemetry_result.get("traces"),
+                    telemetry_result.get("trace_count", 0),
+                    telemetry_result.get("metrics"),
+                )
+
+            # Output result
+            _write_output(
+                uncaptured=uncaptured,
+                captured=captured,
+                sync_result=sync_result,
+                prompt_uncaptured=config.stop_prompt_uncaptured,
+            )
+
+        except json.JSONDecodeError as e:
+            timer.set_status("error")
+            logger.error("Failed to parse hook input: %s", e)
+            print(json.dumps({"continue": True}))
+        except Exception as e:
+            timer.set_status("error")
+            logger.exception("Stop hook error: %s", e)
+            print(json.dumps({"continue": True}))
+        finally:
+            cancel_timeout()
 
     sys.exit(0)
 
