@@ -21,10 +21,13 @@ Architecture:
 
 from __future__ import annotations
 
-import contextlib
+import logging
 import sqlite3
+import struct
+import threading
 from contextlib import contextmanager
 from datetime import UTC, datetime
+from functools import lru_cache
 from pathlib import Path
 from typing import TYPE_CHECKING
 
@@ -32,6 +35,29 @@ import sqlite_vec
 
 from git_notes_memory.config import EMBEDDING_DIMENSIONS, get_index_path
 from git_notes_memory.exceptions import MemoryIndexError
+from git_notes_memory.observability.decorators import measure_duration
+from git_notes_memory.observability.metrics import get_metrics
+from git_notes_memory.observability.tracing import trace_operation
+
+logger = logging.getLogger(__name__)
+
+
+# PERF-007: Cache compiled struct format for embedding serialization
+@lru_cache(maxsize=1)
+def _get_struct_format(dimensions: int) -> struct.Struct:
+    """Get a cached struct.Struct for packing embeddings.
+
+    The embedding dimensions are typically constant (384 for all-MiniLM-L6-v2),
+    so caching the compiled Struct avoids repeated format string parsing.
+
+    Args:
+        dimensions: Number of float values in the embedding.
+
+    Returns:
+        A compiled struct.Struct instance for packing.
+    """
+    return struct.Struct(f"{dimensions}f")
+
 
 if TYPE_CHECKING:
     from collections.abc import Iterator, Sequence
@@ -77,6 +103,8 @@ _CREATE_INDICES = [
     "CREATE INDEX IF NOT EXISTS idx_memories_timestamp ON memories(timestamp)",
     "CREATE INDEX IF NOT EXISTS idx_memories_status ON memories(status)",
     "CREATE INDEX IF NOT EXISTS idx_memories_repo_path ON memories(repo_path)",
+    # HIGH-004: Composite index for efficient range queries within namespace
+    "CREATE INDEX IF NOT EXISTS idx_memories_namespace_timestamp ON memories(namespace, timestamp DESC)",
 ]
 
 # Migration SQL for schema version upgrades
@@ -141,6 +169,8 @@ class IndexService:
         self.db_path = db_path or get_index_path()
         self._conn: sqlite3.Connection | None = None
         self._initialized = False
+        # HIGH-011: Thread lock for concurrent access safety
+        self._lock = threading.Lock()
 
     @property
     def is_initialized(self) -> bool:
@@ -171,6 +201,11 @@ class IndexService:
             )
             self._conn.row_factory = sqlite3.Row
 
+            # MED-005: Enable WAL mode for better concurrent access
+            # WAL allows readers and writers to operate concurrently
+            self._conn.execute("PRAGMA journal_mode=WAL")
+            self._conn.execute("PRAGMA synchronous=NORMAL")
+
             # Load sqlite-vec extension
             self._load_vec_extension()
 
@@ -269,8 +304,16 @@ class IndexService:
 
             # Create indices (ignore if they already exist)
             for index_sql in _CREATE_INDICES:
-                with contextlib.suppress(sqlite3.OperationalError):
+                try:
                     cursor.execute(index_sql)
+                except sqlite3.OperationalError as e:
+                    # Index likely already exists - this is expected on subsequent inits
+                    logger.debug("Index creation skipped (already exists): %s", e)
+                    metrics = get_metrics()
+                    metrics.increment(
+                        "silent_failures_total",
+                        labels={"location": "index.create_index_skipped"},
+                    )
 
             # Create vector table
             cursor.execute(_CREATE_VEC_TABLE)
@@ -334,6 +377,7 @@ class IndexService:
     # Insert Operations
     # =========================================================================
 
+    @measure_duration("index_insert")
     def insert(
         self,
         memory: Memory,
@@ -361,16 +405,20 @@ class IndexService:
             )
 
         now = datetime.now(UTC).isoformat()
+        metrics = get_metrics()
 
-        with self._cursor() as cursor:
+        with (
+            trace_operation("index.insert", labels={"namespace": memory.namespace}),
+            self._cursor() as cursor,
+        ):
             try:
                 # Insert into memories table
                 cursor.execute(
                     """
                     INSERT INTO memories (
                         id, commit_sha, namespace, summary, content,
-                        timestamp, repo_path, spec, phase, tags, status, relates_to,
-                        created_at, updated_at
+                        timestamp, repo_path, spec, phase, tags, status,
+                        relates_to, created_at, updated_at
                     ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
                     """,
                     (
@@ -396,6 +444,12 @@ class IndexService:
                     self._insert_embedding(cursor, memory.id, embedding)
 
                 self._conn.commit()  # type: ignore[union-attr]
+
+                metrics.increment(
+                    "index_inserts_total",
+                    labels={"namespace": memory.namespace},
+                )
+
                 return True
 
             except sqlite3.IntegrityError as e:
@@ -447,13 +501,14 @@ class IndexService:
             try:
                 for i, memory in enumerate(memories):
                     try:
+                        # CRIT-002: Include repo_path for per-repository isolation
                         cursor.execute(
                             """
                             INSERT INTO memories (
                                 id, commit_sha, namespace, summary, content,
-                                timestamp, spec, phase, tags, status, relates_to,
-                                created_at, updated_at
-                            ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
+                                timestamp, repo_path, spec, phase, tags, status,
+                                relates_to, created_at, updated_at
+                            ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
                             """,
                             (
                                 memory.id,
@@ -462,6 +517,7 @@ class IndexService:
                                 memory.summary,
                                 memory.content,
                                 memory.timestamp.isoformat(),
+                                memory.repo_path,
                                 memory.spec,
                                 memory.phase,
                                 ",".join(memory.tags) if memory.tags else None,
@@ -508,10 +564,8 @@ class IndexService:
             memory_id: ID of the memory this embedding belongs to.
             embedding: The embedding vector.
         """
-        # sqlite-vec expects binary format for vectors
-        import struct
-
-        blob = struct.pack(f"{len(embedding)}f", *embedding)
+        # PERF-007: Use cached struct format for embedding packing
+        blob = _get_struct_format(len(embedding)).pack(*embedding)
         cursor.execute(
             "INSERT INTO vec_memories (id, embedding) VALUES (?, ?)",
             (memory_id, blob),
@@ -684,6 +738,31 @@ class IndexService:
             cursor.execute("SELECT id FROM memories")
             return [row[0] for row in cursor.fetchall()]
 
+    def get_all_memories(
+        self,
+        namespace: str | None = None,
+    ) -> list[Memory]:
+        """Get all memories in the index.
+
+        Args:
+            namespace: Optional namespace filter.
+
+        Returns:
+            List of all Memory objects.
+        """
+        query = "SELECT * FROM memories WHERE 1=1"
+        params: list[object] = []
+
+        if namespace is not None:
+            query += " AND namespace = ?"
+            params.append(namespace)
+
+        query += " ORDER BY timestamp DESC"
+
+        with self._cursor() as cursor:
+            cursor.execute(query, params)
+            return [self._row_to_memory(row) for row in cursor.fetchall()]
+
     def exists(self, memory_id: str) -> bool:
         """Check if a memory exists in the index.
 
@@ -820,9 +899,8 @@ class IndexService:
             memory_id: ID of the memory this embedding belongs to.
             embedding: The new embedding vector.
         """
-        import struct
-
-        blob = struct.pack(f"{len(embedding)}f", *embedding)
+        # PERF-007: Use cached struct format for embedding packing
+        blob = _get_struct_format(len(embedding)).pack(*embedding)
 
         # Delete existing and insert new (sqlite-vec doesn't support UPDATE well)
         cursor.execute("DELETE FROM vec_memories WHERE id = ?", (memory_id,))
@@ -962,6 +1040,7 @@ class IndexService:
     # Search Operations
     # =========================================================================
 
+    @measure_duration("index_search_vector")
     def search_vector(
         self,
         query_embedding: Sequence[float],
@@ -984,50 +1063,56 @@ class IndexService:
             List of (Memory, distance) tuples sorted by distance ascending.
             Lower distance means more similar.
         """
-        import struct
+        metrics = get_metrics()
 
-        # Pack query embedding as binary
-        blob = struct.pack(f"{len(query_embedding)}f", *query_embedding)
+        with trace_operation("index.search_vector", labels={"k": str(k)}):
+            # PERF-007: Use cached struct format for embedding packing
+            blob = _get_struct_format(len(query_embedding)).pack(*query_embedding)
 
-        with self._cursor() as cursor:
-            try:
-                # Build parameterized query with optional filters
-                # Use single JOIN to eliminate N+1 query pattern
-                params: list[object] = [blob, k * 3]
+            with self._cursor() as cursor:
+                try:
+                    # Build parameterized query with optional filters
+                    # Use single JOIN to eliminate N+1 query pattern
+                    params: list[object] = [blob, k * 3]
 
-                sql = """
-                    SELECT m.*, v.distance
-                    FROM vec_memories v
-                    JOIN memories m ON v.id = m.id
-                    WHERE v.embedding MATCH ?
-                      AND k = ?
-                """
+                    sql = """
+                        SELECT m.*, v.distance
+                        FROM vec_memories v
+                        JOIN memories m ON v.id = m.id
+                        WHERE v.embedding MATCH ?
+                          AND k = ?
+                    """
 
-                if namespace is not None:
-                    sql += " AND m.namespace = ?"
-                    params.append(namespace)
-                if spec is not None:
-                    sql += " AND m.spec = ?"
-                    params.append(spec)
+                    if namespace is not None:
+                        sql += " AND m.namespace = ?"
+                        params.append(namespace)
+                    if spec is not None:
+                        sql += " AND m.spec = ?"
+                        params.append(spec)
 
-                sql += " ORDER BY v.distance LIMIT ?"
-                params.append(k)
+                    sql += " ORDER BY v.distance LIMIT ?"
+                    params.append(k)
 
-                cursor.execute(sql, params)
+                    cursor.execute(sql, params)
 
-                results: list[tuple[Memory, float]] = []
-                for row in cursor.fetchall():
-                    memory = self._row_to_memory(row)
-                    distance = row["distance"]
-                    results.append((memory, distance))
+                    results: list[tuple[Memory, float]] = []
+                    for row in cursor.fetchall():
+                        memory = self._row_to_memory(row)
+                        distance = row["distance"]
+                        results.append((memory, distance))
 
-                return results
+                    metrics.increment(
+                        "index_searches_total",
+                        labels={"search_type": "vector"},
+                    )
 
-            except Exception as e:
-                raise MemoryIndexError(
-                    f"Vector search failed: {e}",
-                    "Check embedding dimensions and retry",
-                ) from e
+                    return results
+
+                except Exception as e:
+                    raise MemoryIndexError(
+                        f"Vector search failed: {e}",
+                        "Check embedding dimensions and retry",
+                    ) from e
 
     def search_text(
         self,
