@@ -20,6 +20,7 @@ from typing import TYPE_CHECKING
 from git_notes_memory.config import NAMESPACES, get_project_index_path
 from git_notes_memory.exceptions import RecallError
 from git_notes_memory.models import Memory, NoteRecord, VerificationResult
+from git_notes_memory.observability.metrics import get_metrics
 
 if TYPE_CHECKING:
     from git_notes_memory.embedding import EmbeddingService
@@ -219,6 +220,8 @@ class SyncService:
         Iterates through all namespaces, lists notes, and parses
         their content into NoteRecord objects.
 
+        Uses batch git operations (PERF-001) for efficient retrieval.
+
         Returns:
             List of all NoteRecord objects from git notes.
         """
@@ -233,9 +236,16 @@ class SyncService:
                 logger.debug("No notes in namespace %s: %s", namespace, e)
                 continue
 
+            if not notes_list:
+                continue
+
+            # PERF-001: Batch fetch all notes for this namespace
+            commit_shas = [commit_sha for _note_sha, commit_sha in notes_list]
+            contents = git_ops.show_notes_batch(namespace, commit_shas)
+
             for _note_sha, commit_sha in notes_list:
                 try:
-                    content = git_ops.show_note(namespace, commit_sha)
+                    content = contents.get(commit_sha)
                     if content:
                         # Pass commit_sha and namespace so NoteRecord has them
                         records = parser.parse_many(
@@ -257,6 +267,9 @@ class SyncService:
     def reindex(self, *, full: bool = False) -> int:
         """Rebuild the index from git notes.
 
+        Uses batch git operations (PERF-001) and batch embedding (PERF-002)
+        for efficient retrieval and vectorization.
+
         Args:
             full: If True, clears index first. Otherwise incremental.
 
@@ -269,7 +282,7 @@ class SyncService:
         """
         git_ops = self._get_git_ops()
         index = self._get_index()
-        embedding = self._get_embedding_service()
+        embedding_service = self._get_embedding_service()
         parser = self._get_note_parser()
 
         if full:
@@ -284,9 +297,20 @@ class SyncService:
                 logger.debug("No notes in namespace %s: %s", namespace, e)
                 continue
 
+            if not notes_list:
+                continue
+
+            # PERF-001: Batch fetch all notes for this namespace
+            commit_shas = [commit_sha for _note_sha, commit_sha in notes_list]
+            contents = git_ops.show_notes_batch(namespace, commit_shas)
+
+            # First pass: collect all memories and texts for batch embedding
+            memories_to_index: list[Memory] = []
+            texts_to_embed: list[str] = []
+
             for _note_sha, commit_sha in notes_list:
                 try:
-                    content = git_ops.show_note(namespace, commit_sha)
+                    content = contents.get(commit_sha)
                     if not content:
                         continue
 
@@ -300,28 +324,8 @@ class SyncService:
                         if not full and index.exists(memory.id):
                             continue
 
-                        # Generate embedding
-                        embed_vector = None
-                        try:
-                            text = f"{memory.summary}\n{memory.content}"
-                            embed_vector = embedding.embed(text)
-                        except Exception as e:
-                            logger.warning(
-                                "Embedding failed for %s: %s",
-                                memory.id,
-                                e,
-                            )
-
-                        # Insert into index
-                        try:
-                            index.insert(memory, embedding=embed_vector)
-                            indexed += 1
-                        except Exception as e:
-                            logger.warning(
-                                "Failed to index memory %s: %s",
-                                memory.id,
-                                e,
-                            )
+                        memories_to_index.append(memory)
+                        texts_to_embed.append(f"{memory.summary}\n{memory.content}")
 
                 except Exception as e:
                     logger.warning(
@@ -331,6 +335,34 @@ class SyncService:
                         e,
                     )
 
+            if not memories_to_index:
+                continue
+
+            # PERF-002: Batch generate all embeddings at once
+            embeddings: list[list[float]] | list[None] = []
+            try:
+                embeddings = embedding_service.embed_batch(texts_to_embed)
+            except Exception as e:
+                logger.warning(
+                    "Batch embedding failed for namespace %s: %s",
+                    namespace,
+                    e,
+                )
+                # Fall back to None embeddings for all
+                embeddings = [None] * len(memories_to_index)
+
+            # Second pass: insert memories with their embeddings
+            for memory, embed_vector in zip(memories_to_index, embeddings, strict=True):
+                try:
+                    index.insert(memory, embedding=embed_vector)
+                    indexed += 1
+                except Exception as e:
+                    logger.warning(
+                        "Failed to index memory %s: %s",
+                        memory.id,
+                        e,
+                    )
+
         logger.info("Reindex complete: %d memories indexed", indexed)
         return indexed
 
@@ -340,6 +372,8 @@ class SyncService:
         Compares the set of memory IDs in the index with those
         that should exist based on git notes content.
 
+        Uses batch git operations (PERF-001) for efficient retrieval.
+
         Returns:
             VerificationResult with details of any inconsistencies.
         """
@@ -357,9 +391,16 @@ class SyncService:
             except Exception:
                 continue
 
+            if not notes_list:
+                continue
+
+            # PERF-001: Batch fetch all notes for this namespace
+            commit_shas = [commit_sha for _note_sha, commit_sha in notes_list]
+            contents = git_ops.show_notes_batch(namespace, commit_shas)
+
             for _note_sha, commit_sha in notes_list:
                 try:
-                    content = git_ops.show_note(namespace, commit_sha)
+                    content = contents.get(commit_sha)
                     if not content:
                         continue
 
@@ -390,6 +431,7 @@ class SyncService:
 
         # Check for content mismatches (simplified - just check if exists)
         mismatched: list[str] = []
+        metrics = get_metrics()
         for memory_id in expected_ids & indexed_ids:
             try:
                 memory = index.get(memory_id)
@@ -401,8 +443,16 @@ class SyncService:
                     expected_hash = memory_hashes.get(memory_id, "")
                     if current_hash != expected_hash:
                         mismatched.append(memory_id)
-            except Exception:
-                pass
+            except Exception as e:
+                logger.warning(
+                    "Failed to verify hash for memory %s: %s",
+                    memory_id,
+                    e,
+                )
+                metrics.increment(
+                    "silent_failures_total",
+                    labels={"location": "sync.hash_verification"},
+                )
 
         is_consistent = (
             len(missing_in_index) == 0
@@ -469,13 +519,45 @@ class SyncService:
         logger.info("Repair complete: %d changes made", repairs)
         return repairs
 
+    # =========================================================================
+    # Remote Sync Operations
+    # =========================================================================
+
+    def sync_with_remote(
+        self,
+        *,
+        namespaces: list[str] | None = None,
+        push: bool = True,
+    ) -> dict[str, bool]:
+        """Synchronize notes with remote and reindex.
+
+        This is the primary entry point for remote synchronization. It:
+        1. Calls GitOps.sync_notes_with_remote() for fetch→merge→push workflow
+        2. Reindexes the local SQLite index after successful merges
+
+        Args:
+            namespaces: Specific namespaces to sync, or None for all.
+            push: Whether to push changes to remote.
+
+        Returns:
+            Dict mapping namespace to sync success.
+        """
+        git_ops = self._get_git_ops()
+
+        # Perform remote sync (fetch → merge → push)
+        results = git_ops.sync_notes_with_remote(namespaces, push=push)
+
+        # Reindex after successful sync to include fetched memories
+        if any(results.values()):
+            self.reindex()
+
+        return results
+
 
 # =============================================================================
-# Singleton Pattern
+# Singleton Access (using ServiceRegistry)
 # =============================================================================
 
-_sync_service: SyncService | None = None
-
 
 def get_sync_service(repo_path: Path | None = None) -> SyncService:
     """Get or create the singleton SyncService instance.
@@ -486,7 +568,10 @@ def get_sync_service(repo_path: Path | None = None) -> SyncService:
     Returns:
         The SyncService singleton.
     """
-    global _sync_service  # noqa: PLW0603
-    if _sync_service is None:
-        _sync_service = SyncService(repo_path)
-    return _sync_service
+    from git_notes_memory.registry import ServiceRegistry
+
+    # If repo_path is provided, pass it to get() for initialization
+    if repo_path is not None:
+        return ServiceRegistry.get(SyncService, repo_path=repo_path)
+
+    return ServiceRegistry.get(SyncService)
